from web3 import Web3
from typing import Optional, Any

def get_network_status(provider_url: str) -> Any:
    w3 = Web3(Web3.HTTPProvider(provider_url))
    return {
        'is_connected': w3.is_connected(),
        'client_version': w3.client_version,
        'chain_id': w3.eth.chain_id
    }

def get_block_number(provider_url: str) -> Optional[int]:
    w3 = Web3(Web3.HTTPProvider(provider_url))
    return w3.eth.block_number

def get_block_details(block_number: int, provider_url: str) -> Any:
    w3 = Web3(Web3.HTTPProvider(provider_url))
    return w3.eth.get_block(block_number)

def get_latest_transactions(provider_url: str, count: int = 10) -> Any:
    w3 = Web3(Web3.HTTPProvider(provider_url))
    latest = w3.eth.block_number
    block = w3.eth.get_block(latest, full_transactions=True)
    return block.transactions[:count] 