from web3 import Web3
from eth_account import Account
import re
from typing import Optional
import secrets
import os

# Create a new Ethereum wallet (private key + address)
def create_wallet() -> dict:
    acct = Account.create(os.urandom(32))
    return {'private_key': acct.key.hex(), 'address': acct.address}

# Validate an Ethereum address
def validate_address(address: str) -> bool:
    return Web3.is_address(address)

# Get the ETH balance of an address
def get_balance(address: str, provider_url: str) -> Optional[float]:
    w3 = Web3(Web3.HTTPProvider(provider_url))
    if not w3.is_connected():
        raise ConnectionError('Web3 provider not connected')
    balance_wei = w3.eth.get_balance(address)
    return w3.from_wei(balance_wei, 'ether')

# Generate a random 12-word mnemonic (BIP39)
def generate_mnemonic() -> str:
    # Uses secrets for randomness, but for real BIP39 use 'mnemonic' package
    import hashlib
    import random
    wordlist_path = os.path.join(os.path.dirname(__file__), 'bip39_wordlist.txt')
    if not os.path.exists(wordlist_path):
        # fallback: use a small built-in list
        wordlist = [
            'apple', 'banana', 'cat', 'dog', 'elephant', 'fish', 'grape', 'hat', 'ice', 'jungle', 'kite', 'lemon',
            'monkey', 'nose', 'orange', 'pear', 'queen', 'rose', 'sun', 'tree', 'umbrella', 'violet', 'wolf', 'xray', 'yellow', 'zebra'
        ]
    else:
        with open(wordlist_path, 'r') as f:
            wordlist = [w.strip() for w in f.readlines() if w.strip()]
    return ' '.join(random.choice(wordlist) for _ in range(12)) 